﻿Type=Class
Version=5.8
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
#Region Class Notes
' Custom View Class - ButtonToolbar
' @autor rwblinn.de
' @since see Class_Globals Version

'Button Properties
'id: Unique ID
'text: Text to be displayed. Do not define when using an icon
'tooltiptext: Tooltip to be displayed
'height: The prefheight = 30
'width: The prefwidth = 100
'icon: The icon must be added to the B4J IDE Files Manager

'Hint: when using an icon, set the width and the height equal, i.e. 32 or 16 ...

'Toolbar Properties
'Buttons: JSON String defining the Buttons (See below example)
'Transparent: True | false to set all Buttons transparency
'Vertical: if true, the Toolbar Orientation is vertical.When anchors in the designer are used, then ensure to set Vertical to both if required.

'Example DesignerProperty Buttons with two buttons Exit (as Icon Button) and About (as Text Button) plus a separator.
'[
'{"id":"exit","width":30,"height":30,"tooltiptext":"Close this App","icon":"exit.png"},
'{"text":"-"},
'{"id":"about","text":"About"}
']
#End Region

' Events
#Event:Action
' Designer
#DesignerProperty: Key:Buttons, Displayname: Buttons, Fieldtype: String, Defaultvalue: , Description: The definition of the buttons.
#DesignerProperty: Key:Transparent, Displayname: Transparent, Fieldtype: Boolean, Defaultvalue: True, Description: Set the buttons to transparent.
#DesignerProperty: Key:Vertical, Displayname: Vertical, Fieldtype: Boolean, Defaultvalue: False, Description: Set the toolbar orientation to vertical.

Private Sub Class_Globals
	Private Version As String = "v20170725"
	Private fx As JFX
	Private mEventName As String 'ignore
	Private mCallBack As Object 'ignore
	Private mBase As Pane
	Private mProps As Map		'ignore
	'Views
	Private mJOToolbar As JavaObject
	Private mButtonPressed As Button
End Sub

Public Sub Initialize (Callback As Object, EventName As String)
	mEventName = EventName
	mCallBack = Callback
End Sub

Public Sub DesignerCreateView (Base As Pane, Lbl As Label, Props As Map)
	' Assign the custom view base to global mBase, the same for the properties
	mBase = Base
	mProps = Props
	' Set properties
	CSSUtils.SetBorder(mBase, 0, fx.Colors.Transparent, 0)
	' Add the buttons
	' Init the toolbar object
	mJOToolbar.InitializeNewInstance("javafx.scene.control.ToolBar", Null)
	If mProps.Get("Vertical") = True Then
		mJOToolbar.RunMethod("setOrientation", Array("VERTICAL"))
	Else
		mJOToolbar.RunMethod("setOrientation", Array("HORIZONTAL"))
	End If
	' Create list of buttons / separators to be added to the toolbar node
	Dim l As List
	l.Initialize
	Dim parser As JSONParser
	parser.Initialize(mProps.Get("Buttons"))
	Dim root As List = parser.NextArray
	' Create a button or separator
	For Each colroot As Map In root
		' Check if separator = "-"
		Dim septext As String = colroot.Get("text")
		' Separator
		If septext.EqualsIgnoreCase("-") Then
			Dim joSep As JavaObject
			' Create new instance depending orientation
			If mProps.Get("Vertical") = True Then
				joSep.InitializeNewInstance("javafx.scene.control.Separator", Array("VERTICAL"))
			Else
				joSep.InitializeNewInstance("javafx.scene.control.Separator", Array("HORIZONTAL"))
			End If
			' Add the separator to the list
			l.Add(joSep)
		End If
		' Button
		If Not(septext.EqualsIgnoreCase("-")) Then
			Dim b As Button
			b.Initialize("ToolBarButton")
			If colroot.Get("id") <> Null Then b.Id = colroot.Get("id")
			If colroot.Get("text") <> Null Then b.Text = colroot.Get("text")
			If colroot.Get("tooltiptext") <> Null Then b.TooltipText = colroot.Get("tooltiptext")
			b.PrefHeight = 30
			If colroot.Get("height") <> Null Then b.PrefHeight = colroot.Get("height")
			b.PrefWidth = 100
			If colroot.Get("width") <> Null Then b.PrefWidth = colroot.Get("width")
			If mProps.Get("Transparent") = True Then CSSUtils.SetStyleProperty(b, "-fx-background-color", "transparent")
			If colroot.Get("icon") <> Null Then
				' Handle fontawesome: ref=https://www.b4x.com/android/forum/threads/b4x-fontawesome-material-icons-fonts.72908/#content
				Dim ic As String = colroot.Get("icon")
				' Fontawesome icon is hex string, i.e. 0x. 
				'Add to the button tag, which is used in base_resize to set the fontawesome icon
				If ic.StartsWith("0x") Then
					b.Tag = ic
					b.Font = fx.CreateFontAwesome(b.PrefHeight)
					b.Text = Chr(Bit.ParseInt(ic.Replace("0x",""), 16))
					b.Alignment = "CENTER_LEFT"
				Else
				' Icon from file located in the dirassets folder
					AddButtonIcon(b, colroot.Get("icon"), mProps.Get("Transparent"))
				End If
			End If
			' Add the button to the list
			l.Add(b)
		End If
	Next
	' Add the list of buttons to the toolbar
	mJOToolbar.RunMethodJO("getItems", Null).RunMethod("addAll", Array(l))
	' Add the toolbar node to the base
	mBase.AddNode(mJOToolbar, 0, 0, 0, 0)
End Sub

' Handle resize of the base which is the toolbar.
Private Sub Base_Resize (Width As Double, Height As Double)
	mJOToolbar.RunMethod("setPrefWidth", Array(Width))
	mJOToolbar.RunMethod("setPrefHeight", Array(Height))
'	Dim l As List = mJOToolbar.RunMethodJO("getItems", Null)
'	For Each n As Node In l
'		If n Is Button Then
'			Dim b As Button = n
'			If b.Id.StartsWith("0x") Then
'				b.Alignment = "CENTER_LEFT"
'			End If
'		End If
'	Next

End Sub

Public Sub GetBase As Pane
	Return mBase
End Sub

' Get or set the id of the view
Public Sub setId(Id As String)
	mBase.Id = Id
End Sub

Public Sub getId As String
	Return mBase.Id
End Sub

' Handle button pressed.
Public Sub getButtonPressed As Button
	Return mButtonPressed
End Sub

Public Sub setButtonPressed(b As Button)
	mButtonPressed = b
End Sub

#Region EVENTS
' Handle Toolbar Button pressed
Private Sub ToolBarButton_Action
	setButtonPressed(Sender)
'	Log("ToolBar Button Clicked = " & b.Tag)
	If SubExists(mCallBack,mEventName & "_Action") Then
		CallSub(mCallBack, mEventName & "_Action")
	End If
End Sub
#End Region

' Get the version of the custom view
Public Sub GetVersion As String
	Return Version
End Sub

#Region Helpers
' Add an icon to a button. The icon transparency is set using flag transparent. The position (=gravity) is center.
' The icon must be located in the assets folder and added to the B4J IDE Files Manager.
Private Sub AddButtonIcon(btn As Button, icon As String, transparent As Boolean)
	If transparent Then CSSUtils.SetStyleProperty(btn, "-fx-background-color", "transparent")
	CSSUtils.SetStyleProperty(btn, "-fx-background-position","center")
	CSSUtils.SetStyleProperty(btn, "-fx-background-repeat", "no-repeat")
	Dim iconpath As String = File.GetUri(File.DirAssets, icon)
	CSSUtils.SetStyleProperty(btn, "-fx-background-image", $"url('${iconpath}');"$)
End Sub
#End Region
