﻿Type=Class
Version=5.8
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
#Region Class Notes
' Custom View Class - ColorList
' @autor rwblinn.de
' @since see Class_Globals Version
#End Region

#Event: SelectedIndexChanged(Index As Int)

Private Sub Class_Globals
	Private Version As String = "v20170201"
	Private fx As JFX
	Private mEventName As String 'ignore
	Private mCallBack As Object 'ignore
	Private mBase As Pane
	Private mProps As Map
	' The views added to the base pane
	Private InlineJava As JavaObject = Me
	Private ColorMap As Map
	Private lvColors As ListView
	Private ColorName As String
	Private ColorValue As String
End Sub

Public Sub Initialize (vCallback As Object, vEventName As String)
	mEventName = vEventName
	mCallBack = vCallback
End Sub

Public Sub DesignerCreateView (Base As Pane, lbl As Label, Props As Map)
	mBase = Base
	mProps = Props
'	CSSUtils.SetBorder(mBase, 0, fx.Colors.Transparent, 0)
	lvColors.Initialize("lvColors")
	lvColors.Style = lbl.Style
	mBase.AddNode(lvColors, 0, 0, 0, 0)
	CallSubDelayed(Me, "ListViewColorList")
End Sub

Public Sub AddToParent(Parent As Pane, Left As Int, Top As Int, Width As Int, Height As Int)
	mBase.Initialize("Base")
	mProps.Initialize
	lvColors.Initialize("lvColors")
	
	mBase.AddNode(lvColors, 0, 0, 0, 0)
	' Add the base node to the parent
	Parent.AddNode(mBase, Left, Top, Width, Height)
	' Resize
	Base_Resize (Width, Height)
	'
	CallSubDelayed(Me, "ListViewColorList")
End Sub

Private Sub Base_Resize (Width As Double, Height As Double)
	lvColors.SetLayoutAnimated(0, 0, 0, Width, Height)
End Sub

Public Sub GetBase As Pane
	Return mBase
End Sub

' Routine: Get or set the width of the base
Public Sub SetWidth(Width As Double)
	If Width < 100 Then Width = 100
	mBase.PrefWidth = Width
End Sub

Public Sub GetWidth As Double
	Return mBase.PrefWidth
End Sub

' Routine: Get or set the height of the base
Public Sub SetHeigth(Height As Double)
	If Height < 10 Then Height = 10
	mBase.PrefHeight=Height
End Sub

Public Sub GetHeigth As Double
	Return mBase.PrefHeight
End Sub

' Property: Get or set the top position of the base
Public Sub setTop(Top As Double)
	If Top < 0 Then Top = 0
	mBase.Top = Top
End Sub

Public Sub getTop As Double
	Return mBase.Top
End Sub

' Property: Get or set the left position of the base
Public Sub setLeft(Left As Double)
	If Left < 0 Then Left = 0
	mBase.Left = Left
End Sub

Public Sub getLeft  As Double
	Return mBase.Left
End Sub

' Property: Get or set the tag of the base
Public Sub setTag(Tag As String)
    mBase.Tag = Tag
End Sub

Public Sub getTag As String
    Return mBase.Tag
End Sub

' Property: Get the Color Name
Public Sub getName As String
	Return ColorName
End Sub

' Property: Get the Color Value
Public Sub getValue As String
	Return ColorValue
End Sub

Public Sub setToolTipText(Value As String)
	lvColors.TooltipText = Value
End Sub

Public Sub getToolTipText As String
	Return lvColors.TooltipText
End Sub

' Property: Get or set the visibility of the base
Public Sub setVisible(Visible As Boolean)
   mBase.Visible = Visible
End Sub

Public Sub getVisible As Boolean
   Return mBase.Visible
End Sub

' Get or set the style.
' Example styles: -fx-font-size:15,00;-fx-text-fill:blue;-fx-underline:true;-fx-cursor:hand;
Public Sub getStyle As String
	Return lvColors.style
End Sub

Public Sub setStyle(Value As String)
	lvColors.style = Value
End Sub

' Get the style class list to add, remove classes.
Public Sub StyleClasses As List
	Return lvColors.StyleClasses
End Sub

' Gets or sets the selected index
Public Sub setSelectedIndex(Index As Int)
	lvColors.SelectedIndex = Index
End Sub

Public Sub getSelectedIndex As Int
	Return lvColors.SelectedIndex
End Sub

' Get the selected color as map "ColorName":"Name", "ColorValue":"0x..."
Public Sub getSelectedItem As Map
	Dim index As Int = getSelectedIndex
	If index < 0 Then Return Null
	' Get the selected anchorpane
	Dim ap As AnchorPane = lvColors.items.Get(index)
	' Get the label node with index 1 = the second item is taken as the first is a pane showing the color.
	Dim lbl As Label = ap.GetNode(1)
	Dim m As Map = CreateMap("ColorName":lbl.Text, "ColorValue":lbl.Tag)
	Return m
End Sub

' Routine: Scroll to item position in the list
Public Sub ScrollTo(Index As Int)
	lvColors.ScrollTo(Index)
End Sub

' Build the listview with list of colors
Private Sub ListViewColorList
	Dim joColors As JavaObject
	joColors.InitializeStatic("javafx.scene.paint.Color")
	ColorMap = InlineJava.RunMethod("GetColorMap", Null)
	For i = 0 To ColorMap.Size - 1   
		ListViewAddRow(lvColors, ColorMap.GetKeyAt(i), ColorMap.GetValueAt(i))
   	Next
End Sub

' Add customized row pane label
Private Sub ListViewAddRow(lv As ListView, Line1 As String, Value As Object)
	Dim ap As AnchorPane
  	ap.Initialize("")
  	' Add a pane showing the color
  	Dim p As Pane
  	p.Initialize("")
	CSSUtils.SetBackgroundColor(p, GetColorFromRGBA(Value))
  	ap.AddNode(p, 0, 0, lv.Width * 0.25, 20)
	' Add a label showing the color hex value
  	Dim lbl1 As Label
  	lbl1.Initialize("")
  	lbl1.Text = Line1                  
  	lbl1.Font = fx.DefaultFont(16)
	lbl1.Tag = Value
  	ap.AddNode(lbl1, lv.Width * 0.25, 0, lv.Width * 0.75, 20)
	' Add the anchorpane to the listview
  	lv.Items.Add(ap)
End Sub

' Event: SelectedIndexChanged
Private Sub lvColors_SelectedIndexChanged(Index As Int)
	If Index < 0 Then Return
	' Get the selected anchorpane
	Dim ap As AnchorPane = lvColors.items.Get(Index)
	' Get the label node with index 1 = the second item is taken as the first is a pane showing the color.
	Dim lbl As Label = ap.GetNode(1)
	' Show the colorname and its value
	ColorName = lbl.Text
	ColorValue = lbl.Tag
	If SubExists(mCallBack,mEventName & "_SelectedIndexChanged") Then
		CallSub2(mCallBack, mEventName & "_SelectedIndexChanged", Index)
	End If
End Sub

' Convert the hex color 0x..... to paint
Private Sub GetColorFromRGBA(hex As String) As Paint
  Dim a, r,g,b As Int
  hex = hex.Replace("0x", "").Replace("#", "")
  a = Bit.ParseInt(hex.SubString2(6,8), 16)
  r = Bit.ParseInt(hex.SubString2(0,2), 16)
  g = Bit.ParseInt(hex.SubString2(2,4), 16)
  b = Bit.ParseInt(hex.SubString2(4,6), 16)
  Return fx.Colors.ARGB(a, r, g, b)
End Sub
 
#If Java
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;
import javafx.scene.paint.Color;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.objects.collections.Map;                                        

  /**
   * GetColorMap
   * @param None 
   * @return Map (B4A object MyMap) of Colors Name = Value
   */
  public static  Map.MyMap GetColorMap() throws IllegalArgumentException, IllegalAccessException, ClassNotFoundException
  {
    Class<?> clcolor = Class.forName("javafx.scene.paint.Color");
      Map.MyMap colormap = new Map.MyMap();
      if (clcolor != null) {
          Field[] field = clcolor.getFields();
          for (int i = 0; i < field.length; i++) {
              Field f = field[i];                
              Object objn = f.get(null);
              if(objn instanceof Color){
                Color c = (Color) objn;
                Map m = new Map();
                m.Initialize();
                m.Put(f.getName(), c.toString());
                colormap.putAll(m.getObject());
              }            
          }
      }
      return colormap;
  }

#End If

' Get the version of the custom view
Public Sub GetVersion As String
	Return Version
End Sub

