﻿Type=Class
Version=5.8
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
#Region Class Notes
' Custom View Class - LinkLabel
' @autor rwblinn.de
' @since see Class_Globals Version
#End Region

' Events
#Event: MouseClicked (EventData As MouseEvent)
#Event: MouseMoved (EventData As MouseEvent)
#Event: MouseDragged (EventData As MouseEvent)
#Event: MousePressed (EventData As MouseEvent)
#Event: MouseReleased (EventData As MouseEvent)
#Event: MouseChanged (EventData As MouseEvent)
#Event: FocusChanged (HasFocus As Boolean)
' Designer Properties
#DesignerProperty: Key:Link, Displayname: Link, Fieldtype: String, Defaultvalue: http://www.somelink.com, Description: Define the link url

Private Sub Class_Globals
	Private Version As String = "v20170201"
	Private fx As JFX
	Private mEventName As String 'ignore
	Private mCallBack As Object 'ignore
	Private mBase As Pane
	Private mProps As Map
	' The view added to the base pane
	Private lblLinkLabel As Label
End Sub

Public Sub Initialize (vCallback As Object, vEventName As String)
	mEventName = vEventName
	mCallBack = vCallback
End Sub

Public Sub DesignerCreateView (Base As Pane, lbl As Label, Props As Map)
	mBase = Base
	mProps = Props
	CSSUtils.SetBorder(mBase, 0, fx.Colors.Transparent, 0)

	lblLinkLabel.Initialize("lblLinkLabel")
	lblLinkLabel.Style = lbl.Style
	CSSUtils.SetStyleProperty(lblLinkLabel, "-fx-text-fill", "blue")
	CSSUtils.SetStyleProperty(lblLinkLabel, "-fx-underline", True)
	CSSUtils.SetStyleProperty(lblLinkLabel, "-fx-cursor", "hand")
	mBase.AddNode(lblLinkLabel, 0, 0, 0, 0)
End Sub

Public Sub AddToParent(Parent As Pane, Left As Int, Top As Int, Width As Int, Height As Int)
	mBase.Initialize("Base")
	mProps.Initialize
	mProps.Put("Link", "http://www.somelink.com")

	lblLinkLabel.Initialize("lblLinkLabel")
	CSSUtils.SetStyleProperty(lblLinkLabel, "-fx-text-fill", "blue")
	CSSUtils.SetStyleProperty(lblLinkLabel, "-fx-underline", True)
	CSSUtils.SetStyleProperty(lblLinkLabel, "-fx-cursor", "hand")
	mBase.AddNode(lblLinkLabel, 0, 0, 0, 0)
	' Add the base node to the parent
	Parent.AddNode(mBase, Left, Top, Width, Height)
	' Resize
	Base_Resize (Width, Height)
End Sub

#Region BASE

Private Sub Base_Resize (Width As Double, Height As Double)
	lblLinkLabel.SetLayoutAnimated(0, 0, 0, Width, Height)
End Sub

' Get the view as pane
Public Sub GetBase As Pane
	Return mBase
End Sub

' Get or set the width of the view
Public Sub SetWidth(Width As Double)
	If Width < 100 Then Width = 100
	mBase.PrefWidth = Width
End Sub

Public Sub GetWidth As Double
	Return mBase.PrefWidth
End Sub

' Get or set the height of the view
Public Sub SetHeigth(Height As Double)
	If Height < 10 Then Height = 10
	mBase.PrefHeight=Height
End Sub

Public Sub GetHeigth As Double
	Return mBase.PrefHeight
End Sub

' Get or set the top position of the view
Public Sub setTop(Top As Double)
	If Top < 0 Then Top = 0
	mBase.Top = Top
End Sub

Public Sub getTop As Double
	Return mBase.Top
End Sub

' Get or set the left position of the view
Public Sub setLeft(Left As Double)
	If Left < 0 Then Left = 0
	mBase.Left = Left
End Sub

Public Sub getLeft  As Double
	Return mBase.Left
End Sub

' Get or set the tag of the view
Public Sub setTag(Tag As String)
    mBase.Tag = Tag
End Sub

Public Sub getTag As String
    Return mBase.Tag
End Sub

' Get or set text of the label
Public Sub getText As String
	Return lblLinkLabel.Text
End Sub

Public Sub setText(Value As String)
	lblLinkLabel.Text = Value
End Sub

' Get or set the label tooltip text
Public Sub setToolTipText(Value As String)
	lblLinkLabel.TooltipText = Value
End Sub

Public Sub getToolTipText As String
	Return lblLinkLabel.TooltipText
End Sub

' Get or set the visibility of the view
Public Sub setVisible(Visible As Boolean)
   mBase.Visible = Visible
End Sub

Public Sub getVisible As Boolean
   Return mBase.Visible
End Sub

' Get or set the text fill style (=the color) of the label.
Public Sub getTextFillStyle As String
	Return CSSUtils.GetStyleProperty(lblLinkLabel, "-fx-text-fill")
End Sub

Public Sub setTextFillStyle(Value As String)
	CSSUtils.SetStyleProperty(lblLinkLabel, "-fx-text-fill", Value)
End Sub

' Get or set the style.
' Example styles: -fx-font-size:15,00;-fx-text-fill:blue;-fx-underline:true;-fx-cursor:hand;
Public Sub getStyle As String
	Return lblLinkLabel.style
End Sub

Public Sub setStyle(Value As String)
	lblLinkLabel.style = Value
End Sub

' Get the style class list to add, remove classes.
Public Sub StyleClasses As List
	Return lblLinkLabel.StyleClasses
End Sub
#End Region

' Get or set the url of the linklabel
Public Sub getLink As String
	Return 	mProps.Get("Link")
End Sub

Public Sub setLink(Value As String)
	mProps.Put("Link", Value)
End Sub

#Region EVENTS
' Handle the label resoze event
Private Sub lblLinkLabel_Resize (Width As Double, Height As Double)
	If SubExists(mCallBack,mEventName & "_Resize") Then
		CallSub3(mCallBack, mEventName & "_Resize", Width, Height)
	End If
End Sub

' Handle mouse click by opening the link in the default browser
Private Sub lblLinkLabel_MouseClicked (EventData As MouseEvent)
	fx.ShowExternalDocument(getLink)
	If SubExists(mCallBack,mEventName & "_MouseClicked") Then
		CallSub2(mCallBack, mEventName & "_MouseClicked", EventData)
	End If
End Sub

' Handle the label mouse moved event
Private Sub lblLinkLabel_MouseMoved (EventData As MouseEvent)
	If SubExists(mCallBack,mEventName & "_MouseMoved") Then
		CallSub2(mCallBack, mEventName & "_MouseMoved", EventData)
	End If
End Sub

' Handle the label mouse dragged event
Private Sub lblLinkLabel_MouseDragged (EventData As MouseEvent)
	If SubExists(mCallBack,mEventName & "_MouseDragged") Then
		CallSub2(mCallBack, mEventName & "_MouseDragged", EventData)
	End If
End Sub

' Handle the label mouse pressed event
Private Sub lblLinkLabel_MousePressed (EventData As MouseEvent)
	If SubExists(mCallBack,mEventName & "_MousePressed") Then
		CallSub2(mCallBack, mEventName & "_MousePressed", EventData)
	End If
End Sub

' Handle the label mouse released event
Private Sub lblLinkLabel_MouseReleased (EventData As MouseEvent)
	If SubExists(mCallBack,mEventName & "_MouseReleased") Then
		CallSub2(mCallBack, mEventName & "_MouseReleased", EventData)
	End If
End Sub

' Handle the label mouse changed event
Private Sub lblLinkLabel_MouseChanged (EventData As MouseEvent)
	If SubExists(mCallBack,mEventName & "_MouseChanged") Then
		CallSub2(mCallBack, mEventName & "_MouseChanged", EventData)
	End If
End Sub

' Handle the label focus changed event
Private Sub lblLinkLabel_FocusChanged (HasFocus As Boolean)
	If SubExists(mCallBack,mEventName & "_FocusChanged") Then
		CallSub2(mCallBack, mEventName & "_FocusChanged", HasFocus)
	End If
End Sub
#End Region

' Get the version of the custom view
Public Sub GetVersion As String
	Return Version
End Sub
