﻿Type=Class
Version=5.8
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
#Region Class Notes
' Custom View Class - ListEdit
' @autor rwblinn.de
' @since see Class_Globals Version
' References:
' JavaFX API TextField: docs.oracle.com/javase/8/javafx/api/javafx/scene/control/TextField.html
' JavaFX API MouseEvent: https://docs.oracle.com/javafx/2/api/javafx/scene/input/MouseEvent.html
#End Region

' Events
#Event: SelectedIndexChanged(Index As Int)
#Event: Resize (Width As Double, Height As Double)
#Event: MouseClicked (EventData As MouseEvent)
#Event: MouseMoved (EventData As MouseEvent)
#Event: MouseDragged (EventData As MouseEvent)
#Event: MousePressed (EventData As MouseEvent)
#Event: MouseReleased (EventData As MouseEvent)
#Event: MouseChanged (EventData As MouseEvent)
#Event: FocusChanged (HasFocus As Boolean)
' Designer Properties
#DesignerProperty: Key: ItemHeight, DisplayName: Item Height, FieldType: Float, DefaultValue: 30, Description: Set the Item Height.
#DesignerProperty: Key: ItemStyle, DisplayName: Item Style, FieldType: String, DefaultValue: -fx-text-fill: Blue; -fx-font-size: 1.1em;, Description: Set the Item Height.

Sub Class_Globals
	Private Version As String = "v20161024"
	Private fx As JFX
	Private mEventName As String 'ignore
	Private mCallBack As Object 'ignore
	Private mBase As Pane
	Private mProps As Map
	' The views added to the base pane
	Private lvEdit As ListView
	Private tfSelected As TextField		' Keeps track of the textfield as part of the selected listview index
	Private ItemID As Int = 0
End Sub

Public Sub Initialize (vCallback As Object, vEventName As String)
	mEventName = vEventName
	mCallBack = vCallback
End Sub

Public Sub DesignerCreateView (Base As Pane, Lbl As Label, Props As Map)
	mBase = Base
	mProps = Props
	CSSUtils.SetBorder(mBase, 0, fx.Colors.Transparent, 0)
	lvEdit.Initialize("lvEdit")
	lvEdit.Style = Lbl.Style
	lvEdit.MultipleSelection = False
	mBase.AddNode(lvEdit, 0, 0, -1, -1)
End Sub

Public Sub AddToParent(Parent As Pane, Left As Int, Top As Int, Width As Int, Height As Int)
	mBase.Initialize("Base")
	mProps.Initialize
	mProps.Put("ItemHeight", 30)
	mProps.Put("ItemStyle", "-fx-text-fill: Blue; -fx-font-size: 1.1em;")
	CSSUtils.SetBorder(mBase, 0, fx.Colors.Transparent, 0)
	lvEdit.Initialize("lvEdit")
	lvEdit.MultipleSelection = False
	mBase.AddNode(lvEdit, 0, 0, -1, -1)
	' Add the base node to the parent
	Parent.AddNode(mBase, Left, Top, Width, Height)
	' Resize
	Base_Resize (Width, Height)
End Sub

' Handle resize of the base. The listview width is added by 10 to remove the hor scrollbar.
Public Sub Base_Resize (Width As Double, Height As Double)
	lvEdit.SetLayoutAnimated(0, 0, 0, Width, Height)
End Sub

Public Sub GetBase As Pane
	Return mBase
End Sub

' Get or set the width of the view
Public Sub SetWidth(Width As Double)
	If Width < 100 Then Width = 100
	mBase.PrefWidth = Width
End Sub

Public Sub GetWidth As Double
	Return mBase.PrefWidth
End Sub

' Get or set the height of the view
Public Sub SetHeigth(Height As Double)
	If Height < 100 Then Height = 100
	mBase.PrefHeight=Height
End Sub

Public Sub GetHeigth As Double
	Return mBase.PrefHeight
End Sub

' Get or set the top position of the view
Public Sub setTop(Top As Double)
	If Top < 0 Then Top = 0
	mBase.Top = Top
End Sub

Public Sub getTop As Double
	Return mBase.Top
End Sub

' Get or set the left position of the view
Public Sub setLeft(Left As Double)
	If Left < 0 Then Left = 0
	mBase.Left = Left
End Sub

Public Sub getLeft  As Double
	Return mBase.Left
End Sub

' Get or set the tag of the view
Public Sub setTag(Tag As String)
    mBase.Tag = Tag
End Sub

Public Sub getTag As String
    Return mBase.Tag
End Sub

' Get or set the tootip text
Public Sub setToolTipText(Value As String)
	lvEdit.TooltipText = Value
End Sub

Public Sub getToolTipText As String
	Return lvEdit.TooltipText
End Sub

' Get or set the visibility of the view
Public Sub setVisible(Visible As Boolean)
   mBase.Visible = Visible
End Sub

Public Sub getVisible As Boolean
   Return mBase.Visible
End Sub

' Get or set the style.
' Example styles: -fx-font-size:15,00;-fx-text-fill:blue;-fx-underline:true;-fx-cursor:hand;
Public Sub getStyle As String
	Return lvEdit.style
End Sub

Public Sub setStyle(Value As String)
	lvEdit.style = Value
End Sub

' Get the style class list to add, remove classes.
Public Sub StyleClasses As List
	Return lvEdit.StyleClasses
End Sub

#Region LISTROUTINES
' Add a new item which is a textfield.
' An keypressed eventhandler is assigned to handle keys UP (to previous listitem) and DOWN (to next listitem).
Public Sub Add(text As String)
	Dim tf As TextField
	tf.Initialize("TextField")
	ItemID = ItemID + 1
	tf.Tag = ItemID
	tf.Text = text
	tf.Style = mProps.Get("ItemStyle")
	tf.Top = 0
	tf.Left = 0
	tf.PrefWidth = 1	' must be greater 0!
	tf.PrefHeight = mProps.Get("ItemHeight")
	lvEdit.Items.Add(tf)
	lvEdit.ScrollTo(lvEdit.Items.Size - 1)
	lvEdit.SelectedIndex = lvEdit.Items.Size - 1
	Dim joTF As JavaObject = tf
  	Dim oTF As Object = joTF.CreateEvent("javafx.event.EventHandler","TextFieldKeyPressed",False)
  	joTF.RunMethod("setOnKeyPressed",Array(oTF))
  	Dim oTF As Object = joTF.CreateEvent("javafx.event.EventHandler","TextFieldKeyReleased",False)
  	joTF.RunMethod("setOnKeyReleased",Array(oTF))
  	Dim oTF As Object = joTF.CreateEvent("javafx.event.EventHandler","TextFieldMouseClicked",False)
  	joTF.RunMethod("setOnMouseClicked",Array(oTF))
'	TODO: Find solution handling focus changed
'  	Dim oTF As Object = joTF.CreateEvent("javafx.event.EventHandler","TextFieldFocusChanged",False)
'  	joTF.RunMethod("setOnFocusChanged",Array(oTF))
End Sub

Public Sub AddAll(itemlist As List)
	For Each item As String In itemlist
		Add(item)
	Next
End Sub

' Remove an item from the list
Public Sub RemoveAt(Index As Int)
	If Index < 0 Then Return
	lvEdit.Items.RemoveAt(Index)
End Sub

' Remove all items from the list
Public Sub Clear
	lvEdit.Items.Clear
End Sub

' Find an item by text in the list
' When item found, scroll to the listitem
Public Sub Find(Text As String)
	If Text.Length = 0 Then Return
	For i = 0 To lvEdit.Items.Size - 1
		Dim tf As TextField = lvEdit.Items.Get(i) 
		If tf.Text = Text Then
			lvEdit.ScrollTo(i)
			lvEdit.SelectedIndex = i
			Return
		End If
	Next
End Sub

' Find an item by tag in the list
' When item found, scroll to the listitem
Public Sub FindTag(Tag As Int)
	For i = 0 To lvEdit.Items.Size - 1
		Dim tf As TextField = lvEdit.Items.Get(i) 
		If tf.tag = Tag Then
			lvEdit.ScrollTo(i)
			lvEdit.SelectedIndex = i
			Return
		End If
	Next
End Sub

' Get the text of the selected item
Public Sub getText As String
	If tfSelected.IsInitialized Then 
		Return tfSelected.Text
	Else
		Return ""
	End If
End Sub

' Set the text of the selected item
Public Sub setText(Text As String)
	If tfSelected.IsInitialized Then 
		tfSelected.Text = Text
	End If
End Sub

' Get the tag of the selected item
Public Sub getItemTag As String
	If tfSelected.IsInitialized Then 
		Return tfSelected.Tag
	Else
		Return ""
	End If
End Sub

' Set the tag of the selected item
Public Sub setItemTag(Tag As String)
	If tfSelected.IsInitialized Then 
		tfSelected.Tag = Tag
	End If
End Sub

' Get the listview index of an item by finding its tag
Public Sub getIndex(Tag As Int) As Int
	Dim Result As Int = -1
	If Tag < 0 Then Return Result
	For i = 0 To lvEdit.Items.Size - 1
		Dim tf As TextField = lvEdit.Items.Get(i) 
		If tf.Tag = Tag Then 
			Result = i
			Return Result
		End If
	Next
	Return Result
End Sub
#End Region

' Get ot set the selected index
Public Sub setSelectedIndex(Index As Int)
	lvEdit.SelectedIndex = Index
End Sub

Public Sub getSelectedIndex As Int
	Return lvEdit.SelectedIndex
End Sub

' Get ot set the selected item
Public Sub getSelectedItem As Object
	Return lvEdit.SelectedItem
End Sub

' Scroll to item position in the list
Public Sub ScrollTo(Index As Int)
	lvEdit.ScrollTo(Index)
End Sub

#End Region

#Region EXPORT
' Export all items to a string. One item per line.
Public Sub ToString As String
	Dim sb As StringBuilder
	sb.Initialize
	For i = 0 To lvEdit.Items.Size - 1
		Dim tf As TextField = lvEdit.Items.Get(i) 
		sb.Append(tf.Text).Append(CRLF)
	Next
	Return sb.ToString
End Sub

' Export all items as string to the clipboard. One item per line.
Public Sub ToClipboard
	Dim sb As StringBuilder
	sb.Initialize
	For i = 0 To lvEdit.Items.Size - 1
		Dim tf As TextField = lvEdit.Items.Get(i) 
		sb.Append(tf.Text).Append(CRLF)
	Next
	fx.Clipboard.SetString(sb.ToString)
End Sub

' Export all items as string to a file. One item per line.
Public Sub ToFile(folder As String, filename As String)
	Dim sb As StringBuilder
	sb.Initialize
	For i = 0 To lvEdit.Items.Size - 1
		Dim tf As TextField = lvEdit.Items.Get(i) 
		sb.Append(tf.Text).Append(CRLF)
	Next
	File.WriteString(folder, filename, sb.ToString)
End Sub

' Export all items to a list.
Public Sub ToList As List
	Dim l As List
	l.Initialize
	For i = 0 To lvEdit.Items.Size - 1
		Dim tf As TextField = lvEdit.Items.Get(i) 
		l.Add(tf.Text)
	Next
	Return l
End Sub
#End Region

' Sort the list ascending or decending.
Public Sub Sort(Ascending As Boolean)
	Dim tmplist As List = ToList
	tmplist.Sort(Ascending)
	Clear
	AddAll(tmplist)
End Sub

#Region EVENTS
' For textfield tfSelected handle keypressed and keyreleased events
Private Sub TextFieldKeyPressed_Event(EventName As String,Args() As Object) As Object
	Dim Index As Int = lvEdit.selectedindex
	Dim KEvent As Event = Args(0)
  	Dim KEventJO As JavaObject = Args(0)
  	Dim KeyName As String = KEventJO.RunMethodJO("getCode",Null).RunMethod("getName",Null)
	Select KeyName.ToUpperCase
    	Case "DOWN"
			If Index < lvEdit.Items.Size - 1 Then Index = Index + 1
			lvEdit.selectedindex = Index 
		    'Consume the event
		    KEvent.Consume
	    Case "UP"
			If Index > 0 Then Index = Index - 1
			lvEdit.selectedindex = Index 
		    'Consume the event
		    KEvent.Consume
  End Select
  Return False
End Sub

' NOT USED
Private Sub TextFieldKeyReleased_Event(EventName As String,Args() As Object) As Object	'ignore
  Return False
End Sub

' Handle mouseclicked event in a textfield.
' The mouseevent api method getSource is used to get the textfield clicked. The event first argument is used for that.
' Then for the textfield, the tag is used to set the listview selected index.
Private Sub TextFieldMouseClicked_Event(EventName As String,Args() As Object) As Object	'ignore
	If Args.length > 0 Then
	  	Dim KEventJO As JavaObject = Args(0)
  		Dim tf As TextField = KEventJO.RunMethod("getSource",Null)
		FindTag(tf.Tag)
	End If
  	Return False
End Sub

' TODO: Find Solution
'Private Sub TextField_FocusChanged (HasFocus As Boolean)
'	CallSub2(mCallBack, "TextField_FocusChanged", HasFocus)
'End Sub

' Handle listview selection. As the listview contains textfields, the selected item textfield is selected.
Private Sub lvEdit_SelectedIndexChanged(Index As Int)
	If Index < 0 Then Return
	tfSelected = lvEdit.selecteditem
	tfSelected.RequestFocus
	If SubExists(mCallBack,mEventName & "_SelectedIndexChanged") Then
		CallSub2(mCallBack, mEventName & "_SelectedIndexChanged", Index)
	End If
End Sub

' Handle the resizing of the listview
Private Sub lvEdit_Resize (Width As Double, Height As Double)
	If SubExists(mCallBack,mEventName & "_Resize") Then
		CallSub3(mCallBack, mEventName & "_Resize", Width, Height)
	End If
End Sub

' Handle the mouse clicked event
Private Sub lvEdit_MouseClicked (EventData As MouseEvent)
	If SubExists(mCallBack,mEventName & "_MouseClicked") Then
		CallSub2(mCallBack, mEventName & "_MouseClicked", EventData)
	End If
End Sub

' Handle the mouse moved event
Private Sub lvEdit_MouseMoved (EventData As MouseEvent)
	If SubExists(mCallBack,mEventName & "_MouseMoved") Then
		CallSub2(mCallBack, mEventName & "_MouseMoved", EventData)
	End If
End Sub

' Handle the mouse dragged event
Private Sub lvEdit_MouseDragged (EventData As MouseEvent)
	If SubExists(mCallBack,mEventName & "_MouseDragged") Then
		CallSub2(mCallBack, mEventName & "_MouseDragged", EventData)
	End If
End Sub

' Handle the mouse pressed event
Private Sub lvEdit_MousePressed (EventData As MouseEvent)
	If SubExists(mCallBack,mEventName & "_MousePressed") Then
		CallSub2(mCallBack, mEventName & "_MousePressed", EventData)
	End If
End Sub

' Handle the mouse released event
Private Sub lvEdit_MouseReleased (EventData As MouseEvent)
	If SubExists(mCallBack,mEventName & "_MouseReleased") Then
		CallSub2(mCallBack, mEventName & "_MouseReleased", EventData)
	End If
End Sub

' Handle the mouse changed event
Private Sub lvEdit_MouseChanged (EventData As MouseEvent)
	If SubExists(mCallBack,mEventName & "_MouseChanged") Then
		CallSub2(mCallBack, mEventName & "_MouseChanged", EventData)
	End If
End Sub

' Handle the focus changed event
Private Sub lvEdit_FocusChanged (HasFocus As Boolean)
	If SubExists(mCallBack,mEventName & "_FocusChanged") Then
		CallSub2(mCallBack, mEventName & "_FocusChanged", HasFocus)
	End If
End Sub
#End Region

' Get the version of the custom view
Public Sub GetVersion As String
	Return Version
End Sub
