﻿Type=Class
Version=5.8
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
#Region Class Notes
' Custom View Class - NumericTextField
' @autor rwblinn.de
' @since see Class_Globals Version
#End Region

' Events
#Event:TextChanged (Old As String, New As String)
#Event:Action
#Event:MouseClicked (EventData As MouseEvent)
#Event:FocusChanged (HasFocus As Boolean)
' Designer
#DesignerProperty: Key:Value, Displayname: Value, Fieldtype: String, Defaultvalue: , Defaultvalue: 0, Description: The value of the numeric text field.
#DesignerProperty: Key:Prompt, DisplayName: Prompt, FieldType: String, DefaultValue: , Description: The prompt of the numeric text field.
#DesignerProperty: Key:DigitalComma, DisplayName: Digital Comma, FieldType: Boolean, DefaultValue: False, Description: Option to use the , (comma) instead of the . (dot).

Private Sub Class_Globals
	Private Version As String = "v20170211"
	Private fx As JFX
	Private mEventName As String 'ignore
	Private mCallBack As Object 'ignore
	Private mBase As Pane
	Private mProps As Map		'ignore
	'Views
	Private tfNumeric As TextField
	'Flags
	Private ValidNumber As Boolean
End Sub

Public Sub Initialize (vCallback As Object, vEventName As String)
	mEventName = vEventName
	mCallBack = vCallback
	ValidNumber = False
End Sub

Public Sub DesignerCreateView (Base As Pane, Lbl As Label, Props As Map)
	' Assign the custom view base to global mBase, the same for the properties
	mBase = Base
	mProps = Props
	' Set properties
	CSSUtils.SetBorder(mBase, 0, fx.Colors.Transparent, 0)
	'TextField Numeric
	tfNumeric.Initialize("tfNumeric")
	tfNumeric.Style = Lbl.Style
	ValidNumber = False
	mBase.AddNode(tfNumeric, 0, 0, 0, 0)
End Sub

Public Sub AddToParent(Parent As Pane, Left As Int, Top As Int, Width As Int, Height As Int)
	mBase.Initialize("Base")
	mProps.Initialize
	mProps.Put("DigitalComma", True)
	' TextField
	tfNumeric.Initialize("tfNumeric")
	mBase.AddNode(tfNumeric, 0, 0, 0, 0)
	' Add the base node to the parent
	Parent.AddNode(mBase, Left, Top, Width, Height)
	' Resize
	Base_Resize (Width, Height)
End Sub

#Region BASE
Private Sub Base_Resize (Width As Double, Height As Double)
	tfNumeric.SetLayoutAnimated(0, 0, 0, Width, Height)
End Sub

' Get the view as pane
Public Sub GetBase As Pane
	Return mBase
End Sub

' Get or set the width of the view
Public Sub SetWidth(Width As Double)
	If Width < 100 Then Width = 100
	mBase.PrefWidth = Width
End Sub

Public Sub GetWidth As Double
	Return mBase.PrefWidth
End Sub

' Get or set the height of the view
Public Sub SetHeigth(Height As Double)
	If Height < 10 Then Height = 10
	mBase.PrefHeight=Height
End Sub

Public Sub GetHeigth As Double
	Return mBase.PrefHeight
End Sub

' Get or set the top position of the view
Public Sub setTop(Top As Double)
	If Top < 0 Then Top = 0
	mBase.Top = Top
End Sub

Public Sub getTop As Double
	Return mBase.Top
End Sub

' Get or set the left position of the view
Public Sub setLeft(Left As Double)
	If Left < 0 Then Left = 0
	mBase.Left = Left
End Sub

Public Sub getLeft  As Double
	Return mBase.Left
End Sub

' Get or set the tag of the view
Public Sub setTag(Tag As String)
    mBase.Tag = Tag
End Sub

Public Sub getTag As String
    Return mBase.Tag
End Sub

' Get or set the visibility of the view
Public Sub setVisible(Visible As Boolean)
   mBase.Visible = Visible
End Sub

Public Sub getVisible As Boolean
   Return mBase.Visible
End Sub
#End Region

#Region TextField Numeric
' Get or set the numeric value of the textfield
Public Sub getValue As String
	Return tfNumeric.Text
End Sub

Public Sub setValue(Value As String)
	tfNumeric.Text = Value
End Sub

' Get or set value of the textfield tooltip
Public Sub setTooltipText(Value As String)
	tfNumeric.TooltipText = Value
End Sub

Public Sub getTooltipText As String
	Return tfNumeric.TooltipText
End Sub

' Get or set the text fill style.
Public Sub getTextFillStyle As String
	Return CSSUtils.GetStyleProperty(tfNumeric, "-fx-text-fill")
End Sub

Public Sub setTextFillStyle(Value As String)
	CSSUtils.SetStyleProperty(tfNumeric, "-fx-text-fill", Value)
End Sub

' Get or set the style.
' Example styles: -fx-font-size:15,00;-fx-text-fill:blue;-fx-underline:true;-fx-cursor:hand;
Public Sub getStyle As String
	Return tfNumeric.style
End Sub

Public Sub setStyle(Value As String)
	tfNumeric.style = Value
End Sub

' Get the style class list enabling to add, remove classes.
Public Sub StyleClasses As List
	Return tfNumeric.StyleClasses
End Sub

' Get or set the digital comma
Public Sub getDigitalComma As Boolean
	Return 	mProps.Get("DigitalComma")
End Sub

Public Sub setDigitalComma(Value As Boolean)
	mProps.Put("DigitalComma", Value)
End Sub

' Checks if the number is valid
Public Sub IsValidNumber As Boolean
	Return ValidNumber
End Sub
#End Region

#Region EVENTS
' Handle TextChanged input. 
' If the digitalcomma option is set, then replace , with . to use IsNumber
' The border color is used as validation indicator, e.g. border is red if input is non-numeric.
Private Sub tfNumeric_TextChanged (Old As String, New As String)
	If New.Length = 0 Then 
		CSSUtils.SetBorder(Sender, 2, fx.Colors.Transparent, 2)
		Return
	End If
	If getDigitalComma Then
		If New.Contains(",") Then New = New.Replace(",", ".")
	End If
	If Not(IsNumber(New)) Then 
		CSSUtils.SetBorder(Sender, 2, fx.Colors.Red, 2)
	Else
		CSSUtils.SetBorder(Sender, 2, fx.Colors.Transparent, 2)
	End If
	If SubExists(mCallBack,mEventName & "_TextChanged") Then
		CallSub3(mCallBack, mEventName & "_TextChanged", Old, New)
	End If
End Sub

' Handle TextField Enter Key pressed
Private Sub tfNumeric_Action
	If SubExists(mCallBack,mEventName & "_Action") Then
		CallSub(mCallBack, mEventName & "_Action")
	End If
End Sub

' Handle TextField Mouse Click event
Private Sub tfNumeric_MouseClicked (EventData As MouseEvent)
	If SubExists(mCallBack,mEventName & "_MouseClicked") Then
		CallSub2(mCallBack, mEventName & "_MouseClicked", EventData)
	End If
End Sub

' Handle TextField Focus Changed event
Private Sub tfNumeric_FocusChanged (HasFocus As Boolean)
	If SubExists(mCallBack,mEventName & "_FocusChanged") Then
		CallSub2(mCallBack, mEventName & "_FocusChanged", HasFocus)
	End If
End Sub
#End Region

' Get the version of the custom view
Public Sub GetVersion As String
	Return Version
End Sub
