﻿Type=Class
Version=5.8
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
#Region Class Notes
' Custom View Class - SliderX
' @autor rwblinn.de
' @since see Class_Globals Version
' References:
' @ref: https://docs.oracle.com/javase/8/javafx/api/javafx/scene/control/Slider.html
' Notes:
' This custom view can not be created via code - the JavaFX method lookup as used by ThumbIcon, TrackColor does not work.
' If multiple sliders are required, then create a layout with only the custom view and add it multiple times.
 #End Region

' Events
#Event: ValueChange (Value As Double)
' Designer Proerties
#DesignerProperty: Key: Value, DisplayName: Value, FieldType: Float, DefaultValue: 0, Description: Set the value.
#DesignerProperty: Key: MinValue, DisplayName: Min Value, FieldType: Float, DefaultValue: 0, Description: Set the min value.
#DesignerProperty: Key: MaxValue, DisplayName: Max Value, FieldType: Float, DefaultValue: 100, Description: Set the max value.
#DesignerProperty: Key: BlockIncrement, DisplayName: Block Increment, FieldType: Float, DefaultValue: 10, Description: Set the block increment.
#DesignerProperty: Key: MajorTickUnit, DisplayName: Major Tick Unit, FieldType: Float, DefaultValue: 10, Description: Set the major tick unit (must be greater 0).
#DesignerProperty: Key: MinorTickCount, DisplayName: Minor Tick Count, FieldType: Int, DefaultValue: 5, Description: Set the minor tick count.
#DesignerProperty: Key: ShowTickLabels, DisplayName: Show Tick Labels, FieldType: Boolean, DefaultValue: True, Description: Set the show tick labels.
#DesignerProperty: Key: ShowTickMarks, DisplayName: Show Tick Marks, FieldType: Boolean, DefaultValue: True, Description: Set the show tick marks.
#DesignerProperty: Key: ValueChanging, DisplayName: Value Changing, FieldType: Boolean, DefaultValue: True, Description: Set the value changing.
#DesignerProperty: Key: SnapToTicks, DisplayName: Snap to Ticks, FieldType: Boolean, DefaultValue: True, Description: Set the snap to ticks.
#DesignerProperty: Key: Vertical, DisplayName: Vertical, FieldType: Boolean, DefaultValue: False, Description: Set the vertical orientation.
#DesignerProperty: Key: ShowThumbValue, DisplayName: Show Thumb Value, FieldType: Boolean, DefaultValue: False, Description: Show the slider value in the thumb.
#DesignerProperty: Key: ThumbValueFormat, DisplayName: Thumb Value Format, FieldType: String, DefaultValue: %.0f, Description: The format of the slider value in the thumb.
#DesignerProperty: Key: ThumbValueStyle, DisplayName: Thumb Value Style, FieldType: String, DefaultValue: , Description: The style of the slider value in the thumb.

Private Sub Class_Globals
	Private Version As String = "v20170210"
	Private fx As JFX
	Private mEventName As String 'ignore
	Private mCallBack As Object 'ignore
	Private mBase As Pane
	Private mProps As Map
	' The views added to the base pane
	Private mSlider As Slider
	Private mSliderObj As JavaObject
End Sub

Public Sub Initialize (vCallback As Object, vEventName As String)
	mEventName = vEventName
	mCallBack = vCallback
End Sub

Public Sub DesignerCreateView (Base As Pane, lbl As Label, Props As Map)
	mBase = Base
	CSSUtils.SetBorder(mBase, 0, fx.Colors.Transparent, 0)
	mProps = Props
	mSlider.Initialize("mSlider")
	mSlider.Style = lbl.Style
	mSliderObj = mSlider
	mBase.AddNode(mSlider, 0, 0, 0, 0)
	' It is not possible to load a layout file directly from DesignerCreateView.
	CallSubDelayed(Me, "SetProperties")
End Sub

Private Sub Base_Resize (Width As Double, Height As Double)
	mSlider.SetLayoutAnimated(0, 0, 0, Width, Height)
End Sub

' Get the view as pane
Public Sub GetBase As Pane
	Return mBase
End Sub

Private Sub SetProperties
	setValue(mProps.Get("Value"))
	setMinValue(mProps.Get("MinValue"))
	setMaxValue(mProps.Get("MaxValue"))
	setBlockIncrement(mProps.Get("BlockIncrement"))
	setMajorTickUnit(mProps.Get("MajorTickUnit"))
	setMinorTickCount(mProps.Get("MinorTickCount"))
	setShowTickLabels(mProps.Get("ShowTickLabels"))
	setShowTickMarks(mProps.Get("ShowTickMarks"))
	setValueChanging(mProps.Get("ValueChanging"))
	setSnapToTicks(mProps.Get("SnapToTicks"))
	setVertical(mProps.Get("Vertical"))
	setShowThumbValue(mProps.Get("ShowThumbValue"))
	setThumbValueFormat(mProps.Get("ThumbValueFormat"))
	setThumbValueStyle(mProps.Get("ThumbValueStyle"))
End Sub

' Get or set the width of the view
Public Sub SetWidth(Width As Double)
	If Width < 100 Then Width = 100
	mBase.PrefWidth = Width
End Sub

Public Sub GetWidth As Double
	Return mBase.PrefWidth
End Sub

' Get or set the height of the view
Public Sub SetHeigth(Height As Double)
	If Height < 10 Then Height = 10
	mBase.PrefHeight=Height
End Sub

Public Sub GetHeigth As Double
	Return mBase.PrefHeight
End Sub

' Get or set the top position of the view
Public Sub setTop(Top As Double)
	If Top < 0 Then Top = 0
	mBase.Top = Top
End Sub

Public Sub getTop As Double
	Return mBase.Top
End Sub

' Get or set the left position of the view
Public Sub setLeft(Left As Double)
	If Left < 0 Then Left = 0
	mBase.Left = Left
End Sub

Public Sub getLeft  As Double
	Return mBase.Left
End Sub

' Get or set the tag of the view
Public Sub setTag(Tag As String)
    mBase.Tag = Tag
End Sub

Public Sub getTag As String
    Return mBase.Tag
End Sub

' Get or set the tooltip text
Public Sub setToolTipText(Value As String)
	mSlider.TooltipText = Value
End Sub

Public Sub getToolTipText As String
	Return mSlider.TooltipText
End Sub

' Get or set the visibility of the view
Public Sub setVisible(Visible As Boolean)
   mBase.Visible = Visible
End Sub

Public Sub getVisible As Boolean
   Return mBase.Visible
End Sub

' Get or set the Slider Block Increment
Public Sub setBlockIncrement(Value As Double)
	mSliderObj.RunMethod("setBlockIncrement", Array(Value))	' Must be double
End Sub	
	
Public Sub getBlockIncrement As Double
	Return mSliderObj.RunMethod("getBlockIncrement", Null)
End Sub	

' Get or set the Slider Major Tick Unit. Must be > 0. If <= 0 then set to 1.
Public Sub setMajorTickUnit(Value As Double)
	If Value <= 0 Then Value = 1
	mSliderObj.RunMethod("setMajorTickUnit", Array(Value))	' Must be double
End Sub	

Public Sub getMajorTickUnit As Double
	Return mSliderObj.RunMethod("getMajorTickUnit", Null)
End Sub	

' Get or set the Slider Minor Tick Count
Public Sub setMinorTickCount(Value As Int)
	mSliderObj.RunMethod("setMinorTickCount", Array(Value))	' Must be double
End Sub	

Public Sub getMinorTickCount As Int
	Return mSliderObj.RunMethod("getMinorTickCount", Null)
End Sub	

' Get or set the Slider Show Tick Labels
Public Sub setShowTickLabels(Value As Boolean)
	mSliderObj.RunMethod("setShowTickLabels", Array(Value))	' Must be double
	mProps.Put("ShowTickLabels", Value)
End Sub	

Public Sub getShowTickLabels As Boolean
	Return 	mProps.Get("ShowTickLabels")
End Sub	

' Get or set the Slider Show Tick Marks
Public Sub setShowTickMarks(Value As Boolean)
	mSliderObj.RunMethod("setShowTickMarks", Array(Value))	' Must be double
	mProps.Put("ShowTickMarks", Value)
End Sub	

Public Sub getShowTickMarks As Boolean
	Return 	mProps.Get("ShowTickMarks")
End Sub	

' Get or set the Slider Value Changing
Public Sub setValueChanging(Value As Boolean)
	mSliderObj.RunMethod("setValueChanging", Array(Value))	' Must be double
	mProps.Put("ValueChanging", Value)
End Sub	

Public Sub getValueChanging As Boolean
	Return mProps.Get("ValueChanging")
End Sub	

' Get or set the Slider Snap to Ticks
Public Sub setSnapToTicks(Value As Boolean)
	mSliderObj.RunMethod("setSnapToTicks", Array(Value))	' Must be double
End Sub	

Public Sub getSnapToTicks As Boolean
	Return mSliderObj.RunMethod("isSnapToTicks", Null)
End Sub	

' Get or set the option of showing the slider value in the thumb of the slider
Public Sub setShowThumbValue(Value As Boolean)
	mProps.Put("ShowThumbValue", Value)
	' If the value is shown in the thumb, bind a label to the thumb which is updated while moving the slider
	If Value Then
    	Dim lblValue As Label
	    lblValue.Initialize("lblvalue")
		lblValue.Style = mProps.Get("ThumbValueStyle")
	    Dim joSlider As JavaObject = mSlider
	    Dim joLabel As JavaObject = lblValue
	    ' Get the valueproperty of the slider as StringBinding
	    Dim vp As JavaObject = joSlider.RunMethodJO("valueProperty", Null).RunMethod("asString", Array(mProps.Get("ThumbValueFormat")))
	    ' Bind the slider stringbinding to the label
	    joLabel.RunMethodjo("textProperty", Null).runmethodjo("bind", Array(vp))
	    ' Add the label to the slider thumb
	    joSlider.RunMethodJo("lookup", Array(".thumb")).RunMethodJo("getChildren", Null).RunMethod("add", Array(lblValue))
	End If
End Sub	

Public Sub getShowThumbValue As Boolean
	Return mProps.Get("ShowThumbValue")
End Sub	

' Get or set the format of the slider value in the thumb of the slider
Public Sub getThumbValueFormat As String
	Return mProps.Get("ThumbValueFormat")
End Sub	

Public Sub setThumbValueFormat(Value As String)
	mProps.Put("ThumbValueFormat", Value)
End Sub	

' Get or set the style of the slider value in the thumb of the slider
Public Sub getThumbValueStyle As String
	Return mProps.Get("ThumbValueStyle")
End Sub	

Public Sub setThumbValueStyle(Value As String)
	mProps.Put("ThumbValueStyle", Value)
End Sub	

' Get or set the Slider value at position
Public Sub getValue As Double
	Return mSlider.Value
End Sub

Public Sub setValue(Value As Double)
	mSlider.Value = Value
End Sub

' Get or set the Slider min value
Public Sub getMinValue As Double
	Return mSlider.MinValue
End Sub

Public Sub setMinValue(Value As Double)
	mSlider.MinValue = Value
End Sub

' Get or set the Slider max value
Public Sub getMaxValue As Double
	Return mSlider.MaxValue
End Sub

Public Sub setMaxValue(Value As Double)
	mSlider.MaxValue = Value
End Sub

' Get or set the Slider vertical orientation
Public Sub getVertical As Boolean
	Return mSlider.Vertical
End Sub

Public Sub setVertical(Value As Boolean)
	mSlider.Vertical = Value
End Sub

' Get or set the text fill style
Public Sub getTextFillStyle As String
	Return CSSUtils.GetStyleProperty(mSlider, "-fx-text-fill")
End Sub

Public Sub setTextFillStyle(Value As String)
	CSSUtils.SetStyleProperty(mSlider, "-fx-text-fill", Value)
End Sub

' Get or set the style.
' Example styles: -fx-font-size:15,00;-fx-text-fill:blue;-fx-underline:true;-fx-cursor:hand;
Public Sub getStyle As String
	Return mSlider.style
End Sub

Public Sub setStyle(Value As String)
	mSlider.style = Value
End Sub

' Get the style class list to add, remove classes.
Public Sub StyleClasses As List
	Return mSlider.StyleClasses
End Sub

' Set the icon of the slider thumb.
' The thumb background and border color are set to transparent. The icon must be located in the files folder.
' Example: ThumbIcon("arrowdown-green.png")
Public Sub ThumbIcon(img As String)
	Dim iv As ImageView
	iv.Initialize("")
	iv.SetImage(fx.LoadImage(File.DirAssets, img))
    mSliderObj.RunMethodJo("lookup", Array(".thumb")).RunMethod("setStyle", Array($"-fx-background-color:transparent;-fx-border-color:transparent;"$))
    mSliderObj.RunMethodJo("lookup", Array(".thumb")).RunMethodJo("getChildren", Null).RunMethod("clear", Null)
    mSliderObj.RunMethodJo("lookup", Array(".thumb")).RunMethodJo("getChildren", Null).RunMethod("add", Array(iv))
End Sub

' Set the color of the slider thumb using RGB values
' Example: ThumbColorRGB(0,255,0)
Public Sub ThumbColorRGB(r As Int, g As Int, b As Int)
	If Not(mSliderObj.IsInitialized) Then Return
    mSliderObj.RunMethodJo("lookup", Array(".thumb")).RunMethod("setStyle", Array($"-fx-background-color:rgb(${r},${g},${b});"$))
End Sub

' Set the style of the slider thumb
' Example: ThumbColorRGB()
Public Sub ThumbStyle(style As String)
	If Not(mSliderObj.IsInitialized) Then Return
    mSliderObj.RunMethodJo("lookup", Array(".thumb")).RunMethod("setStyle", Array(style))
End Sub

' Set the color of the slider track via colorname (red, green, blue ...)
' Example: TrackColorName("blue")
Public Sub TrackColorName(color As String)
    mSliderObj.RunMethodJo("lookup", Array(".track")).RunMethod("setStyle", Array($"-fx-background-color:${color};"$))
End Sub

' Set the color of the slider track using RGB values
' Example: TrackColorRGB(0,255,0)
Public Sub TrackColorRGB(r As Int, g As Int, b As Int)
    mSliderObj.RunMethodJO("lookup", Array(".track")).RunMethod("setStyle", Array($"-fx-background-color:rgb(${r},${g},${b});"$))
End Sub

' The Value Change event. The property Slider.value is set according position.
Private Sub mSlider_ValueChange (Value As Double)
	If SubExists(mCallBack,mEventName & "_ValueChange") Then
		CallSub2(mCallBack, mEventName & "_ValueChange", Value)
	End If
End Sub

' Get the version of the custom view
Public Sub GetVersion As String
	Return Version
End Sub
